from re import compile, IGNORECASE, Pattern
import country_converter as coco


def get_country_regex(country: str) -> Pattern:
    pat = coco.convert(names=country, to="regex", not_found=None)
    if not pat:
        raise ValueError("Unknown country: {}".format(country))

    return compile(pat, IGNORECASE)



from re import compile, IGNORECASE
from xlrd import open_workbook
from xlrd.sheet import Sheet
from xlrd.book import Book
from xlrd.biffh import XLRDError
import requests
from .juniper import get_domain_for as jnpr_domain_for
from .countries import get_country_regex

"""
This module reads a spreadsheet from cisco.com, which is what is houses the data being used on:
https://www.cisco.com/c/dam/assets/prod/wireless/wireless-compliance-tool/index.html
This page is basically javascript, reading data from the spreadsheet, and then offering it in an interactive web form.
This is all nice and fine, but in certain cases it would be neat if you could automate which specific AP model to buy,
given the country.
"""
compliance_url = "https://www.cisco.com/c/dam/assets/prod/wireless/wireless-compliance-tool/ComplianceStatus.xls"
platforms = {
    "Controller-based": (1, 3),
    "Standalone": (0, 1),
    "Outdoor & Industrial": (1, 2),
    "Universal": (0, 1),
}
book = None
models_by_platform = None
model_pat = compile(r"^(AIR-|IW|C9).*", IGNORECASE)
MAX_COL = 160


def parse_row(row: int, cols: dict[str, int], sheet: Sheet) -> dict[str, str]:
    """
    parse a row into a dict
    :param int row: row index
    :param dict cols: dict of header, column index
    :param Sheet sheet: sheet to parse data from
    :return: dict of values key'ed by their column name
    :rtype: dict[str, str]
    """
    vals = {}
    for header, col in cols.items():
        cell = sheet.cell(row, col)
        vals[header] = cell.value
    return vals


def get_book() -> Book:
    """
    Lazily fetch the spreadsheet, fresh from cisco.com
    :return:
    :rtype: Book
    """
    global book
    if not book:
        xls_data = requests.get(compliance_url, allow_redirects=True).content
        book = open_workbook(file_contents=xls_data, on_demand=True)
    return book


def get_models_by_platform() -> dict[str, list[str]]:
    """
    Retrieve all models, arranged as a dict of platforms, each having a list of models
    :return:
    :rtype: dict[str, list]
    """
    global models_by_platform
    global platforms
    if models_by_platform:
        return models_by_platform
    models_by_platform = {}
    for p, indexes in platforms.items():
        models_by_platform[p] = set()
        sheet = get_platform_sheet(p)
        start_col = indexes[1] + 1
        try:
            for col in range(start_col, MAX_COL):
                txt = sheet.cell(0, col).value
                if model_pat.search(txt):
                    models_by_platform[p].add(txt.upper())
                else:
                    break
        except IndexError:
            pass
    return models_by_platform


def get_models() -> list[str]:
    """
    Get all models in a list
    :return:
    :rtype: list[str]
    """
    models = set()
    for platform, platform_models in get_models_by_platform().items():
        for model in platform_models:
            models.add(model)
    return list(models)


def get_platform_sheet(platform: str) -> Sheet:
    """
    :param str platform:
    :rtype: Sheet
    """
    try:
        sheet = get_book().sheet_by_name(platform)
    except XLRDError as sheet_error:
        if "&" not in platform:
            raise sheet_error
        sheet = get_book().sheet_by_name(platform.replace("&", "and"))
    return sheet


def get_domain_for(model: str, country=None) -> list[str]:
    """
    Get all valid domains for a given model, in a particular country.
    :param str model:
    :param str country:
    :return:
    """
    valid_domains = set()
    found_country = False
    if country:
        cpat = get_country_regex(country)

    for platform in get_platform_for(model):
        sheet = get_platform_sheet(platform)
        h = {
            "Country": platforms[platform][0],
            "Regulatory Domain": platforms[platform][1],
        }
        for header, col in h.items():
            if header != sheet.cell(0, col).value:
                raise ValueError(
                    "Found unexpected header at 0,{}: {} vs {}".format(
                        col, sheet.cell(0, col).value, header
                    )
                )
        for col in range(
            platforms[platform][1] + 1, MAX_COL
        ):  # models start just after regulatory domain column
            try:
                if sheet.cell(0, col).value.upper() == model.upper():
                    h[model] = col
                    break
            except IndexError:
                raise ValueError(
                    "Unable to find the model {} in the {} platform?".format(
                        model, platform
                    )
                )
        for row_no in range(1, sheet.nrows):
            row = parse_row(row_no, h, sheet)
            if not row:
                continue
            # this way we also match United States in United States of America
            if not country or cpat.search(row["Country"]):
                found_country = True
                if row[model] == "x":
                    valid_domains.add(row["Regulatory Domain"])

    if valid_domains:
        return list(valid_domains)
    elif found_country:
        raise ValueError(
            f"Found {model} for {country} - but no active regulatory domains?"
        )
    elif country:
        rd = jnpr_domain_for(country)
        return [rd]
    else:
        raise ValueError(f"Couldn't find any country matching {country}")


def get_country_models(model: str) -> list[str]:
    """
    Get all valid domain-specific models for a given model.
    :param str model:
    :return:
    """
    return get_models_for(model)


def get_models_for(model: str, country=None) -> list[str]:
    """
    Get precise model name for a model and country
    :param str model:
    :param str country:
    :return:
    :rtype: list[str]
    """

    domains = get_domain_for(model, country)
    if not domains:
        raise ValueError(
            f"Unable to find any valid regulatory domains for {model} in {country}"
        )

    return [f"{model.upper()}{dom}{'-K9' if model.upper().startswith('AIR-') else ''}" for dom in domains]


def get_platform_for(model: str) -> list[str]:
    """
    Fetch platform and indexes for respectively country and regulatory domain as a tuple
    :param model:
    :return:
    :rtype: (str, int, int)
    """
    global platforms
    models = get_models_by_platform()
    model_platforms = []
    for platform, platform_models in models.items():
        if model in platform_models:
            model_platforms.append(platform)
    if not model_platforms:
        raise ValueError(f"Couldn't find the model {model}")
    return model_platforms



from unittest import TestCase, main
from ciscoaplookup import *


class TestCiscoAPLookup(TestCase):
    def test_positive_models(self):
        cases = [
            ('AIR-CAP1532I', 'Belgium', '-E-K9'),
            ('AIR-AP1562I', 'Gibraltar', '-E-K9'),
            ('AIR-CAP1532I', 'Japan', '-Q-K9'),
            ('AIR-AP1572IC', 'Mexico', '-A-K9'),
            ('AIR-AP1572IC', 'New Zealand', '-Z-K9'),
            ('AIR-CAP1532I', 'Taiwan', '-T-K9'),
            ('AIR-AP2802I', 'Denmark', '-E-K9'),
            ('AIR-AP2802I', 'Nicaragua', '-W-K9'),  # used when no matches are made.
            ('C9136I', 'Denmark', '-E'),
            ('C9120AXI', 'Denmark', '-E')
        ]
        for c in cases:
            res = get_models_for(c[0], c[1])
            exp = [c[0]+c[2]]
            self.assertEqual(exp, res, '{} doesn\'t match {} for {} / {}'.format(res, exp, c[0], c[1]))

    def test_fail_models(self):
        self.assertRaises(ValueError, get_models_for, 'gnyf', 'Denmark')  # invalid model
        self.assertRaises(ValueError, get_models_for, 'AIR-CAP1532I', 'Neverland')  # invalid country
        self.assertRaises(ValueError, get_models_for, 'AIR-CAP1552H', 'Vietnam')  # not possible

    def test_hest(self):
        self.assertTrue(len(get_models()) > 20)

    def test_countries(self):
        import pycountry
        lawless_countries = []

        for c in pycountry.countries:
            try:
                klaf = get_models_for('AIR-AP2802I', c.name)
            except ValueError as ve:
                print(ve)


if __name__ == '__main__':
    main()



import requests
from bs4 import BeautifulSoup
from ciscoaplookup.countries import get_country_regex

compliance_url = (
    "https://www.juniper.net/documentation/en_US/release-independent/junos/topics/reference/"
    "specifications/access-point-ax411-country-channel-support.html"
)

_rd_by_cc = None
_rd_by_cn = None


def _init_rd_maps() -> None:
    global _rd_by_cc
    if _rd_by_cc:
        return
    global _rd_by_cn
    rd_by_cn = {}
    rd_by_cc = {}
    html = requests.get(compliance_url, allow_redirects=True).content
    parsed = BeautifulSoup(html, features="html.parser")
    table = parsed.body.find("table", attrs={"cellspacing": "0"})
    for tr in table.tbody.children:
        strings = list(tr.strings)
        if len(strings) < 4:
            break
        rd = strings[3][-2:]
        rd_by_cc[strings[0]] = rd
        rd_by_cn[strings[1].lower()] = rd

    _rd_by_cn = rd_by_cn
    _rd_by_cc = rd_by_cc


def get_domain_for(country: str) -> str:
    """
    Get valid domain, in a particular country.
    :param str country:
    :return:
    """
    global _rd_by_cn
    _init_rd_maps()
    if country in _rd_by_cn:
        return _rd_by_cn[country.lower()]
    pat = get_country_regex(country)
    for cn, rd in _rd_by_cn.items():
        if pat.search(cn):
            return rd
    raise ValueError("Couldn't find any country matching {}".format(country))


if __name__ == "__main__":
    _init_rd_maps()
    print(get_domain_for("Viet Nam"))



from setuptools import setup, find_packages
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ciscoaplookup',
    version="0.11.1",
    author="Steffen Schumacher",
    author_email="ssch@wheel.dk",
    description="The Cisco Wireless LAN Compliance Lookup library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/steffenschumacher/ciscoaplookup.git",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    install_requires=['requests', 'xlrd==1.2.0', 'beautifulsoup4', 'country_converter'],
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
    ],
)